<?php
/**
 * Includes class to create websafe names
 *
 * @author      Stefan Seifarth / stese
 * @copyright    www.polycoder.de
 * @author      Murat Purc <murat@purc.de>
 * @date        04.12.2004
 * @date        28.12.2005
 * @package     Contenido
 * @subpackage  ModRewrite
 */

/******************************************
 * File      :   class.modrewrite.php
 * Project   :   Contenido
 * Descr     :   class to create websafe names
 *
 * Author    :   Stefan Seifarth / stese
 * Created   :   04.12.2004
 * Modified  :   28.12.2005
 *
 *  www.polycoder.de
 */


defined('CON_FRAMEWORK') or die('Illegal call');


/**
 * Class to create websafe names
 *
 * @author      Stefan Seifarth / stese
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  ModRewrite
 */
class ModRewrite extends ModRewriteBase {

    /**
     * Database instance
     *
     * @var  DB_Contenido
     */
    private static $_db;


    /**
     * Initialization, is to call at least once, also possible to call multible
     * times, if different client configuration is to load.
     *
     * Loads configuration of passed client and sets some properties.
     *
     * @param  int  $clientId  Client id
     */
    public static function initialize($clientId) {
        mr_loadConfiguration($clientId, true);
        parent::initialize();
        self::$_db = new DB_Contenido();
    }


    /**
     * Check categories on websafe name
     *
     * Check all categories in the main parent category on existing same websafe name
     *
     * @param   string  $sName    Websafe name to check
     * @param   int     $iCatId   Current category id
     * @param   int     $iLangId  Current language id
     * @return 	bool    True if websafename already exists, false if not
     */
    public static function isInCategories($sName='', $iCatId=0, $iLangId=0) {
        $sName   = self::$_db->escape($sName);
        $iCatId  = (int) $iCatId;
        $iLangId = (int) $iLangId;

        $aTab = parent::$_oGlobals->get('cfg/tab');

        // get parentid
        $iParentId = 0;
        $sql = "SELECT parentid FROM " . $aTab['cat'] . " WHERE idcat = '$iCatId'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            $iParentId = ($aData['parentid'] > 0 ) ? (int) $aData['parentid'] : 0;
        }

        // check if websafe name is in this category
        $sql = "SELECT count(cl.idcat) as numcats FROM " . $aTab['cat_lang'] . " cl "
             . "LEFT JOIN " . $aTab['cat'] . " c ON cl.idcat = c.idcat WHERE "
             . "c.parentid = '$iParentId' AND cl.idlang = '" . $iLangId . "' AND "
             . "cl.urlname = '" . $sName . "' AND cl.idcat <> '$iCatId'";
        self::$_oDebug->log($sql, 'ModRewrite::isInCategories $sql');

        if ($aData = mr_queryAndNextRecord($sql)) {
            return ($aData['numcats'] > 0) ? true : false;
        }

        return false;
    }


    /**
     * Check articles on websafe name.
     *
     * Check all articles in the current category on existing same websafe name.
     *
     * @param	string  $sName    Websafe name to check
     * @param	int     $iArtId   Current article id
     * @param	int     $iLangId  Current language id
     * @param   int     $iCatId   Category id
     * @return 	bool    True if websafename already exists, false if not
     */
    public static function isInCatArticles($sName='', $iArtId=0, $iLangId=0, $iCatId=0) {
        $sName   = self::$_db->escape($sName);
        $iArtId  = (int) $iArtId;
        $iLangId = (int) $iLangId;
        $iCatId  = (int) $iCatId;

        $aTab = parent::$_oGlobals->get('cfg/tab');

        // handle multipages
        if ($iCatId == 0) {
            // get category id if not set
            $sql = "SELECT idcat FROM " . $aTab['cat_art'] . " WHERE idart = '$iArtId'";
            if ($aData = mr_queryAndNextRecord($sql)) {
                $iCatId = ($aData['idcat'] > 0) ? (int) $aData['idcat'] : 0;
            }
        }

        // check if websafe name is in this category
        $sql = "SELECT count(al.idart) as numcats FROM " . $aTab['art_lang'] . " al "
             . "LEFT JOIN " . $aTab['cat_art'] . " ca ON al.idart = ca.idart WHERE "
             . " ca.idcat='$iCatId' AND al.idlang='" . $iLangId . "' AND "
             . "al.urlname='" . $sName . "' AND al.idart <> '$iArtId'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            return ($aData['numcats'] > 0) ? true : false;
        }

        return false;
    }


    /**
     * Set websafe name in article list.
     *
     * Insert new websafe name in article list
     *
     * @param   string  $sName    Original name (will be converted)
     * @param   int     $iArtId   Current article id
     * @param   int     $iLangId  Current language id
     * @param   int     $iCatId   Category id
     * @return 	bool    True if insert was successfully
     */
    public static function setArtWebsafeName($sName='', $iArtId=0, $iLangId=0, $iCatId=0) {
        $iArtId  = (int) $iArtId;
        $iLangId = (int) $iLangId;
        $iCatId  = (int) $iCatId;

        // get websafe name
        $sNewName = capiStrCleanURLCharacters(html_entity_decode($sName));

        // remove double or more separators
        $sNewName = mr_removeMultipleChars('-', $sNewName);

        $sNewName = self::$_db->escape($sNewName);

        // check if websafe name already exists
        if (self::isInCatArticles($sNewName, $iArtId, $iLangId, $iCatId)) {
            // create new websafe name if exists
            $sNewName = $sNewName . $iArtId;
        }

        // check again - and set name
        if (!self::isInCatArticles($sNewName, $iArtId, $iLangId, $iCatId)) {
            // insert websafe name in article list
            $tab = parent::$_oGlobals->get('cfg/tab/art_lang');
            $sql = "UPDATE " . $tab . " SET urlname = '$sNewName' WHERE idart = '$iArtId' AND idlang = '$iLangId'";
            return self::$_db->query($sql);
        } else {
            return false;
        }
    }


    /**
     * Set websafe name in category list.
     *
     * Insert new websafe name in category list.
     *
     * @param   string  $sName    Original name (will be converted) or alias
     * @param   int     $iCatId   Category id
     * @param   int     $iLangId  Language id
     * @return  bool    True if insert was successfully
     */
    public static function setCatWebsafeName($sName='', $iCatId=0, $iLangId=0) {
        $iCatId  = (int) $iCatId;
        $iLangId = (int) $iLangId;

        // create websafe name
        $sNewName = capiStrCleanURLCharacters(html_entity_decode($sName));

        // remove double or more separators
        $sNewName = mr_removeMultipleChars('-', $sNewName);

        $sNewName = self::$_db->escape($sNewName);

        // check if websafe name already exists
        if (self::isInCategories($sNewName, $iCatId, $iLangId)) {
            // create new websafe name if exists
            $sNewName = $sNewName . $iCatId;
        }

        // check again - and set name
        if (!self::isInCategories($sNewName, $iCatId, $iLangId)) {
            // insert websafe name in article list
            $tag = parent::$_oGlobals->get('cfg/tab/cat_lang');
            $sql = "UPDATE " . $tag . " SET urlname = '$sNewName' WHERE idcat = '$iCatId' AND idlang = '$iLangId'";

            self::$_oDebug->log(array(
                'sName'    => $sName,
                'iCatId'   => $iCatId,
                'iLangId'  => $iLangId,
                'sNewName' => $sNewName
                ), 'ModRewrite::setCatWebsafeName $data'
            );

            return self::$_db->query($sql);

        } else {
            return false;
        }
    }


    /**
     * Set urlpath of category
     *
     * @param   int     $iCatId   Category id
     * @param   int     $iLangId  Language id
     * @return  bool    True if insert was successfully
     */
    public static function setCatUrlPath($iCatId=0, $iLangId=0) {
        $sPath = self::buildRecursivPath($iCatId, $iLangId);

        $iCatId  = (int) $iCatId;
        $iLangId = (int) $iLangId;

        // insert websafe name in article list
        $tab = parent::$_oGlobals->get('cfg/tab/cat_lang');
        $sql = "UPDATE " . $tab . " SET urlpath = '$sPath' WHERE idcat = '$iCatId' AND idlang = '$iLangId'";

        self::$_oDebug->log(array(
            'iCatId'   => $iCatId,
            'iLangId'  => $iLangId,
            'sPath'    => $sPath
            ), 'ModRewrite::setCatUrlPath $data'
        );

        return self::$_db->query($sql);
    }


    /**
     * Get article id and language id from article language id
     *
     * @param   int    $iArtlangId  Current article id
     * @return  array  Array with idart and idlang of current article
     */
    public static function getArtIdByArtlangId($iArtlangId=0) {
        $iArtlangId = (int) $iArtlangId;
        $tab = parent::$_oGlobals->get('cfg/tab/art_lang');
        $sql = "SELECT idart, idlang FROM " . $tab . " WHERE idartlang = '$iArtlangId'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            return $aData;
        }
        return array();
    }


    /**
     * Get article id by article websafe name
     *
     * @param   string  Websafe name
     * @param   int     Category id
     * @return  int     Recent article id
     */
    public static function getArtIdByWebsafeName($sArtName='', $iCatId=0, $iLangId=0) {
        $sArtName = self::$_db->escape($sArtName);
        $iCatId   = (int) $iCatId;
        $iLangId  = (int) $iLangId;

        $aTab = parent::$_oGlobals->get('cfg/tab');

        $iArtId = false;

        $sWhere = '';
        if ($iLangId !== 0) {
            $sWhere = ' AND al.idlang=' . $iLangId;
        }
        // only article name were given
        if ($iCatId == 0) {
            // get all basic category ids with parentid=0
            $aCatIds = array();
            $sql = "SELECT idcat FROM " . $aTab['cat'] . " WHERE parentid = '0'";
            self::$_db->query($sql);
            while (self::$_db->next_record()) {
                $aCatIds[] = "idcat = '" . self::$_db->f('idcat') . "'";
            }
            $sWhere .= " AND ( " . join(" OR ", $aCatIds) . ")";
        } else {
            $sWhere .= " AND ca.idcat = '$iCatId'";
        }

        $sql = "SELECT al.idart FROM " . $aTab['art_lang'] . " al "
             . "LEFT JOIN " . $aTab['cat_art'] . " ca ON al.idart = ca.idart "
             . "WHERE al.urlname = '$sArtName'" . $sWhere;

        if ($aData = mr_queryAndNextRecord($sql)) {
            $iArtId = $aData['idart'];
        }

        return $iArtId;
    }


    /**
     * Get category name from category id and language id.
     *
     * @param   int     $iCatId   Category id
     * @param   int     $iLangId  Language id
     * @return  string  Category name
     */
    public static function getCatName($iCatId=0, $iLangId=0) {
        $iCatId  = (int) $iCatId;
        $iLangId = (int) $iLangId;
        $key     = $iCatId . '-' . $iLangId;

        $key = 'mr_statics/catname_by_catid_idlang/' . $iCatId . '_' . $iLangId;
        $catName = parent::$_oGlobals->get($key);
        if ($catName !== null) {
            return $catName;
        }

        $tab = parent::$_oGlobals->get('cfg/tab/cat_lang');
        $sql = "SELECT name FROM " . $tab . " WHERE idcat = '$iCatId' AND idlang = '$iLangId'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            $catName = $aData['name'];
        } else {
            $catName = '';
        }

        parent::$_oGlobals->set($key, $catName);
        return $catName;
    }


    /**
     * Funcion to return cat id by path.
     *
     * Caches the paths at first call to provode faster processing at further calls.
     *
     * @param   string  $path  Category path
     * @return  int  Category id
     */
    public static function getCatIdByUrlPath($path) {
        if (strpos($path, '/') === 0) {
            $path = substr($path, 1);
        }
        if (strrpos($path, '/') === strlen($path)-1) {
            $path = substr($path, 0, -1);
        }

        $catSeperator = '/';
        $startFromRoot = parent::getConfig('startfromroot');
        $urls2lowercase = parent::getConfig('use_lowercase_uri');

        $path = str_replace('/', parent::getConfig('category_seperator'), $path);

        $key = 'mr_statics/cat_ids_and_urlpath';
        $aPathsCache = parent::$_oGlobals->get($key, array());

        if (count($aPathsCache) == 0) {
            $aTab   = parent::$_oGlobals->get('cfg/tab');
            $client = (int) parent::$_oGlobals->get('client');
            $lang   = (int) parent::$_oGlobals->get('lang');

            $sql = "SELECT cl.idcat, cl.urlpath FROM " . $aTab['cat_lang'] . " AS cl, " . $aTab['cat'] . " AS c WHERE "
                 . "c.idclient = " . $client . " AND c.idcat = cl.idcat AND cl.idlang = " . $lang;
            self::$_db->query($sql);
            while (self::$_db->next_record()) {
                $urlPath = self::$_db->f('urlpath');
                if ($startFromRoot == 0 && strpos($urlPath, $catSeperator) > 0) {
                    // paths are stored with prefixed main category, but created
                    // urls doesn't contain the main cat, remove it...
                    $urlPath = substr($urlPath, strpos($urlPath, $catSeperator)+1);
                }
                if ($urls2lowercase) {
                    $urlPath = strtolower($urlPath);
                }

                // store path
                $aPathsCache[self::$_db->f('idcat')] = $urlPath;
            }
        }
        parent::$_oGlobals->set($key, $aPathsCache);

        // compare paths using the similar_text algorithm
        $fPercent = 0;
        $aResults = array();
        foreach ($aPathsCache as $id => $pathItem) {
            similar_text($path, $pathItem, $fPercent);
            $aResults[$id] = $fPercent;
        }

        arsort($aResults, SORT_NUMERIC);
        reset($aResults);

        parent::$_oDebug->addDebug($path, 'ModRewrite::getCatIdByUrlPath() $path');
        parent::$_oDebug->addDebug($aPathsCache, 'ModRewrite::getCatIdByUrlPath() $aPathsCache');
        parent::$_oDebug->addDebug($aResults, 'ModRewrite::getCatIdByUrlPath() $aResults');

        $iMinPercentage = (int) parent::getConfig('category_resolve_min_percentage', 0);
        $catId = key($aResults);
        if ($iMinPercentage > 0 && $aResults[$catId] < $iMinPercentage) {
            return 0;
        } else {
            return $catId;
        }
    }


    /**
     * Get article name from article id and language id
     *
     * @NOTE: seems to be not used???
     *
     * @param   int     $iArtId   Article id
     * @param   int     $iLangId  Language id
     * @return  string  Article name
     */
    public static function getArtTitle($iArtId=0, $iLangId=0) {
        $iArtId  = (int) $iArtId;
        $iLangId = (int) $iLangId;

        $tab = parent::$_oGlobals->get('cfg/tab/art_lang');

        $sql = "SELECT title FROM " . $tab . " WHERE idart = '$iArtId' AND idlang = '$iLangId'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            return $aData['title'];
        }
        return '';
    }


    /**
     * Get language ids from category id
     *
     * @param   int    $iCatId  Category id
     * @return  array  Used language ids
     */
    public static function getCatLanguages($iCatId=0) {
        $iCatId = (int) $iCatId;

        $key = 'mr_statics/cat_idlang_by_catid/' . $iCatId;
        $aLanguages = parent::$_oGlobals->get($key);
        if ($aLanguages !== null) {
            return $aLanguages;
        }

        $aLanguages = array();

        $tab = parent::$_oGlobals->get('cfg/tab/cat_lang');
        $sql = "SELECT idlang FROM " . $tab . " WHERE idcat = '$iCatId'";
        self::$_db->query($sql);
        while (self::$_db->next_record()) {
            $aLanguages[] = self::$_db->f('idlang');
        }

        parent::$_oGlobals->set($key, $aLanguages);
        return $aLanguages;
    }


    /**
     * Get article urlname and language id
     *
     * @param   int    $iArtlangId  idartlang
     * @return  array  Urlname, idlang of empty array
     */
    public static function getArtIds($iArtlangId=0) {
        $iArtlangId = (int) $iArtlangId;
        $tab = parent::$_oGlobals->get('cfg/tab/art_lang');
        $sql = "SELECT urlname, idlang FROM " . $tab . " WHERE idartlang = '$iArtlangId'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            return $aData;
        }
        return array();
    }


    /**
     * Build a recursiv path for mod_rewrite rule like server directories
     * (dir1/dir2/dir3)
     *
     * @param   int     $iCatId   Latest category id
     * @param   int     $iLangId  Language id
     * @param   int     $iLastId  Last category id
     * @return 	string	linkpath with correct uri
     */
    public static function buildRecursivPath($iCatId=0, $iLangId=0, $iLastId=0) {
        $aDirectories  = array();
        $bFinish       = false;
        $iTmpCatId     = (int) $iCatId;
        $iLangId       = (int) $iLangId;
        $iLastId       = (int) $iLastId;

        $aTabs = parent::$_oGlobals->get('cfg/tab');

        while ($bFinish == false) {
            $sql = "SELECT cl.urlname, c.parentid FROM " . $aTabs['cat_lang'] . " cl "
                 . "LEFT JOIN " . $aTabs['cat'] . " c ON cl.idcat = c.idcat "
                 . "WHERE cl.idcat = '$iTmpCatId' AND cl.idlang = '$iLangId'";
            if ($aData = mr_queryAndNextRecord($sql)) {
                $aDirectories[] = $aData['urlname'];
                $iTmpCatId      = (int) $aData['parentid'];

                if ($aData['parentid'] == 0 || $aData['parentid'] == $iLastId) {
                    $bFinish = true;
                }
            } else {
                $bFinish = true;
            }
        }

        // reverse array entries and create directory string
        $sPath = join('/', array_reverse($aDirectories));

        return $sPath;
    }


    /**
     * Return full contenido url from single anchor
     *
     * @param   array   $aMatches [0] = complete anchor, [1] = pre arguments, [2] = anchor name, [3] = post arguments
     * @return  string  New anchor
     */
    public static function rewriteHtmlAnchor(array $aMatches=array()) {
        $artname = parent::$_oGlobals->get('artname');
        $sess    = parent::$_oGlobals->get('sess');

        // set article name
        $sArtParam = '';
        if (isset($artname) && strlen($artname) > 0) {
            $sArtParam = '&idart=' . parent::$_oGlobals->get('idart');
        }

        // check for additional parameter in url
        $aParamsToIgnore = array ('idcat', 'idart', 'lang', 'client', 'idcatart', 'changelang', 'changeclient', 'idartlang', 'parts', 'artname');
        $sOtherParams = '';

        if (isset($_GET) && count($_GET) > 0) {
            foreach ($_GET as $key => $value) {
                if (!in_array($key, $aParamsToIgnore) && strlen(trim($value)) > 0) {
                    $aNoAnchor     = explode('#', $value);
                    $sOtherParams .= '&' . urlencode(urldecode($key)) . '=' . urlencode(urldecode($value));
                }
            }
        }

        $idcat  = (int) parent::$_oGlobals->get('idcat');
        $client = (int) parent::$_oGlobals->get('client');
        $lang   = (int) parent::$_oGlobals->get('lang');

        $url = $sess->url(
            'front_content.php?' . 'idcat=' . $idcat . '&client=' . $client .
            '&changelang=' . $lang . $sArtParam . $sOtherParams . '#' . $aMatches[2]
        );

        $sNewUrl = '<a' . $aMatches[1] . 'href="' . $url . '"' . $aMatches[3] . '>';

        return $sNewUrl;
    }


    /**
     * Return full contenido url from single anchor
     *
     * @param   array   $aMatches [0] = complete anchor, [1] = pre arguments, [2] = anchor name, [3] = post arguments
     * @return  string  New anchor
     */
    public static function contenidoHtmlAnchor(array $aMatches=array(), $bXHTML=true) {
        $artname = parent::$_oGlobals->get('artname');
        $sess    = parent::$_oGlobals->get('sess');

        $aParams    = array();
        $sAmpersand = $bXHTML ? '&amp;' : '&';

        foreach ($_GET as $key => $value) {
            $aNoAnchor = explode('#', $value);
            $aParams[] = urlencode(urldecode($key)) . '=' . urlencode(urldecode($aNoAnchor[0]));
        }

        $url =  $sess->url( 'front_content.php?' . implode($sAmpersand, $aParams) . '#' . $aMatches[2]);
        $sNewUrl = '<a' . $aMatches[1] . 'href="' . $url . '"' . $aMatches[3] . '>';

        return $sNewUrl;
    }


    /**
     * Get article websafe name from article id and language id.
     *
     * @param	int     $iArtId   Article id
     * @param	int     $iLangId  Language id
     * @return 	string	Article websafe name
     */
    public static function getArtWebsafeName($iArtId=0, $iLangId=0) {
        $iArtId  = (int) $iArtId;
        $iLangId = (int) $iLangId;
        $tab = parent::$_oGlobals->get('cfg/tab/art_lang');
        $sql = "SELECT urlname FROM " . $tab . " WHERE "
             . "idart = '" . $iArtId . "' AND idlang = '" . $iLangId . "'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            return urldecode($aData['urlname']);
        }
        return null;
    }


    /**
     * Get article websafe name from idartlang.
     *
     * @param	int     $iArtLangId  idartlang
     * @return 	string	Article websafe name
     */
    public static function getArtLangWebsafeName($iArtLangId=0) {
        $iArtLangId = (int) $iArtLangId;
        $tab = parent::$_oGlobals->get('cfg/tab/art_lang');
        $sql = "SELECT urlname FROM " . $tab . " WHERE idartlang = '". $iArtLangId . "'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            return urldecode($aData['urlname']);
        }
        return null;
    }


    /**
     * Get name of client by id.
     *
     * @param   int     $clientId  Client id
     * @return  string  Client name
     */
    public static function getClientName($clientId=0) {
        $clientId = (int) $clientId;

        $key = 'mr_statics/clientname_by_clientid/' . $clientId;
        $clientName = parent::$_oGlobals->get($key);
        if ($clientName !== null) {
            return $clientName;
        }

        $tab = parent::$_oGlobals->get('cfg/tab/clients');
        $sql = "SELECT name FROM " . $tab . " WHERE idclient = '$clientId'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            $clientName = $aData['name'];
        } else {
            $clientName = '';
        }
        parent::$_oGlobals->set($key, $clientName);
        return $clientName;
    }


    /**
     * Get client id from client name
     *
     * @param   string   Client name
     * @return  integer  Client id
     */
    public static function getClientId($sClientName='') {
        $sClientName = self::$_db->escape($sClientName);
        $key = 'mr_statics/clientid_by_name/' . $sClientName;
        $clientId = parent::$_oGlobals->get($key);
        if ($clientId !== null) {
            return $clientId;
        }

        $tab = parent::$_oGlobals->get('cfg/tab/clients');
        $sql = "SELECT idclient FROM " . $tab . " WHERE name = '" . urldecode($sClientName) . "'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            $clientId = $aData['idclient'];
        } else {
            $clientId = false;
        }
        parent::$_oGlobals->set($key, $clientId);
        return $clientId;
    }


    /**
     * Returns name of language by id.
     *
     * @param   int     $languageId  Language id
     * @return  string  Lanuage name
     */
    public static function getLanguageName($languageId=0) {
        $languageId = (int) $languageId;

        $key = 'mr_statics/languagename_by_id/' . $languageId;
        $languageName = parent::$_oGlobals->get($key);
        if ($languageName !== null) {
            return $languageName;
        }

        $tab = parent::$_oGlobals->get('cfg/tab/lang');
        $sql = "SELECT name FROM " . $tab . " WHERE idlang = '$languageId'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            $languageName = $aData['name'];
        } else {
            $languageName = '';
        }
        parent::$_oGlobals->set($key, $languageName);
        return $languageName;
    }


    /**
     * Get language id from language name thanks to Nicolas Dickinson for multi
     * Client/Language BugFix
     *
     * @param  string   $sLanguageName  Language name
     * @param  int      $iClientId      Client id
     * @return integer  Language id
     */
    public static function getLanguageId($sLanguageName='', $iClientId=1) {
        $sLanguageName = self::$_db->escape($sLanguageName);
        $iClientId     = (int) $iClientId;

        $key = 'mr_statics/langid_by_langname_clientid/' . $sLanguageName . '_' . $iClientId;
        $languageId = parent::$_oGlobals->get($key);
        if ($languageId !== null) {
            return $languageId;
        }

        $aTab = parent::$_oGlobals->get('cfg/tab');
        $sql  = "SELECT l.idlang FROM " . $aTab['lang'] . " as l "
              . "LEFT JOIN " . $aTab['clients_lang'] . " AS cl ON l.idlang = cl.idlang "
              . "WHERE cl.idclient = '". $iClientId . "' AND l.name = '" . urldecode($sLanguageName) . "'";
        if ($aData = mr_queryAndNextRecord($sql)) {
            $languageId = $aData['idlang'];
        } else {
            $languageId = false;
        }
        parent::$_oGlobals->set($key, $languageId);
        return $languageId;
    }


    /**
     * Splits passed argument into scheme://host and path/query.
     *
     * Example:
     * input  = http://host/front_content.php?idcat=123
     * return = array('htmlpath' => 'http://host', 'url' => 'front_content.php?idcat=123')
     *
     * @param  string  $url  URL to split
     * @return array   Assoziative array including the two parts:
     *                 - array('htmlpath' => $path, 'url' => $url)
     */
    public static function getClientFullUrlParts($url) {
        $path = 'cfgClient/' . parent::$_oGlobals->get('client') . '/path/htmlpath';
        $clientPath = parent::$_oGlobals->get($path);

        if (stristr($url, $clientPath) !== false) {

            // url includes full html path (scheme host path, etc.)
            $url      = str_replace($clientPath, '', $url);
            $htmlPath = $clientPath;
            $aComp    = parse_url($htmlPath);

            // check if path matches to defined rootdir from mod_rewrite conf
            if (isset($aComp['path']) && $aComp['path'] !== parent::getConfig('rootdir')) {
                // replace not matching path agaings configured one
                // this will replace e. g. "http://host/cms/" against "http://host/"
                $htmlPath = str_replace($aComp['path'], parent::getConfig('rootdir'), $htmlPath);
                if (substr($htmlPath, strlen($htmlPath)-1) == '/') {
                    // remove last slash
                    $htmlPath = substr($htmlPath, 0, strlen($htmlPath)-1);
                }
            }
        } else {
            $htmlPath = '';
        }
        return array('htmlpath' => $htmlPath, 'url' => $url);
    }


    /**
     * Function to preclean a url.
     *
     * Removes absolute path declaration '/front_content.php' or relative path
     * definition to actual dir './front_content.php', ampersand entities '&amp;'
     * and returns a url like 'front_content.php?idart=12&idlang=1'
     *
     * @param   string  $url  Url to clean
     * @return  string  Cleaned Url
     */
    public static function urlPreClean($url) {
        // some preparation of different front_content.php occurence
        if (strpos($url, './front_content.php') === 0) {
            $url = str_replace('./front_content.php', 'front_content.php', $url);
        } elseif (strpos($url, '/front_content.php') === 0) {
            $url = str_replace('/front_content.php', 'front_content.php', $url);
        }
        $url = str_replace('&amp;', '&', $url);
        return $url;
    }


    /**
     * Method to reset all aliases in categories.
     *
     * Clears all urlname entries in cat_lang table, and sets the value for all
     * existing entries.
     */
    public static function resetCategoriesAliases() {
        $db = new DB_Contenido();

        $tab = parent::$_oGlobals->get('cfg/tab/cat_lang');

        // empty all aliases
        $db->query("UPDATE " . $tab . " SET urlname = ''");

        $aCats = array();

        // get all categories
        $db->query("SELECT name, idcat, idlang FROM " . $tab);
        while ($db->next_record()) {
            //set new alias
            self::setCatWebsafeName($db->f('name'), $db->f('idcat'), $db->f('idlang'));
            $aCats[] = array('idcat' => $db->f('idcat'), 'idlang' => $db->f('idlang'));
        }

        foreach ($aCats as $p => $item) {
            self::setCatUrlPath($item['idcat'], $item['idlang']);
        }

        unset($db, $aCats);
    }


    /**
     * Method to reset all aliases in articles.
     *
     * Clears all urlname entries in art_lang table, and sets the value for all
     * existing entries.
     */
    public static function resetArticlesAliases() {
        $db = new DB_Contenido();

        $tab = parent::$_oGlobals->get('cfg/tab/art_lang');

        // empty all aliases
        $db->query("UPDATE " . $tab . " SET urlname = ''");

        // get all articles
        $db->query("SELECT title, idart, idlang FROM " . $tab);
        while ($db->next_record()) {
            //set new alias
            self::setArtWebsafeName($db->f('title'), $db->f('idart'), $db->f('idlang'));
        }

        unset ($db);
    }


    /**
     * Method to reset all aliases (categories and articles).
     *
     * Shortcut to resetArticlesAliases() and resetArticlesAliases()
     */
    public static function resetAliases() {
        self::resetCategoriesAliases();
        self::resetArticlesAliases();
    }


    /**
     * Used to postprocess resolved path
     *
     * Error site handling if category not found
     *
     * if percentage == 100 and there is no 100 percentage result value,
     * error site will be shown - can be adjust by user settings for
     * smooth similar effects - 80 to 95 will be best but have to check by user
     *
     * @deprecated Is no more used
     * @todo remove usage at /contenido/includes/functions.pathresolver.php
     *
     * @param   array  $results  Pathresolver results array
     * @return  mixed  Categoryid or false
     */
    public static function getIdFromPathresolverResult($results) {
        $iMinPercentage = (int) parent::getConfig('category_resolve_min_percentage', 0);
        $catId = key($results);
        if ($iMinPercentage > 0 && $results[$catId] < $iMinPercentage) {
            return false;
        } else {
            return $catId;
        }
    }

}
